@@ -237,6 +237,7 @@ fn write_serialize_variable<W: Write>(
                 value_type.as_ref(),
                 FieldType::Scalar(ScalarType::I64)
                     | FieldType::Scalar(ScalarType::U64)
+                    | FieldType::Scalar(ScalarType::Bytes)
                     | FieldType::Enum(_)
             ) =>
         {
@@ -255,6 +256,13 @@ fn write_serialize_variable<W: Write>(
                         Indent(indent + 1)
                     )?;
                 }
+                FieldType::Scalar(ScalarType::Bytes) => {
+                    writeln!(
+                        writer,
+                        "{}.map(|(k, v)| (k, pbjson::private::base64::encode(v))).collect();",
+                        Indent(indent + 1)
+                    )?;
+                }
                 FieldType::Enum(path) => {
                     writeln!(writer, "{}.map(|(k, v)| {{", Indent(indent + 1))?;
                     write!(writer, "{}let v = ", Indent(indent + 2))?;
@@ -432,7 +440,7 @@ fn write_deserialize_message<W: Write>(
 {indent}        formatter.write_str("struct {name}")
 {indent}    }}
 
-{indent}    fn visit_map<V>(self, mut map: V) -> Result<{rust_type}, V::Error>
+{indent}    fn visit_map<V>(self, mut map: V) -> std::result::Result<{rust_type}, V::Error>
 {indent}        where
 {indent}            V: serde::de::MapAccess<'de>,
 {indent}    {{"#,
@@ -555,7 +563,7 @@ fn write_deserialize_field_name<W: Write>(
     writeln!(
         writer,
         r#"{indent}impl<'de> serde::Deserialize<'de> for GeneratedField {{
-{indent}    fn deserialize<D>(deserializer: D) -> Result<GeneratedField, D::Error>
+{indent}    fn deserialize<D>(deserializer: D) -> std::result::Result<GeneratedField, D::Error>
 {indent}    where
 {indent}        D: serde::Deserializer<'de>,
 {indent}    {{
@@ -568,7 +576,7 @@ fn write_deserialize_field_name<W: Write>(
 {indent}                write!(formatter, "expected one of: {{:?}}", &FIELDS)
 {indent}            }}
 
-{indent}            fn visit_str<E>(self, value: &str) -> Result<GeneratedField, E>
+{indent}            fn visit_str<E>(self, value: &str) -> std::result::Result<GeneratedField, E>
 {indent}            where
 {indent}                E: serde::de::Error,
 {indent}            {{"#,
@@ -702,9 +710,8 @@ fn write_deserialize_field<W: Write>(
             )?;
 
             let map_k = match key {
-                ScalarType::Bytes => {
-                    // https://github.com/tokio-rs/prost/issues/531
-                    panic!("bytes are not currently supported as map keys")
+                ScalarType::Bytes | ScalarType::F32 | ScalarType::F64 => {
+                    panic!("protobuf disallows maps with floating point or bytes keys")
                 }
                 _ if key.is_numeric() => {
                     write!(
@@ -730,8 +737,8 @@ fn write_deserialize_field<W: Write>(
                     "v.0"
                 }
                 FieldType::Scalar(ScalarType::Bytes) => {
-                    // https://github.com/tokio-rs/prost/issues/531
-                    panic!("bytes are not currently supported as map values")
+                    write!(writer, "::pbjson::private::BytesDeserialize<_>",)?;
+                    "v.0"
                 }
                 FieldType::Enum(path) => {
                     write!(writer, "{}", resolver.rust_type(path))?;
